/****************************************************************************
 *  
 */
#define VERSION "0.3"
/*
 ***************************************************************************/

#include <avr/io.h>
#include <avr/wdt.h>			/* war: <wdt.h> 	*/
#include <avr/pgmspace.h>		/* war: <progmem.h> 	*/
#include <avr/signal.h>			/* war: <sig-avr.h> 	*/
#include <avr/interrupt.h>		/* war: <interrupt.h> 	*/
#include <ctype.h>

#include <avr/sfr_defs.h>		/* cbi() sbi() */
	
# define portor(p,v)            outb(p, inb(p) | (v))
# define portand(p,v)           outb(p, inb(p) & (v))
# define portin(p)              inb(p)
# define portout(p,v)           outb(p,v)
# define prgmem(a)              PRG_RDB(a)
# define clrpbit(p,b)           cbi(p, b)
# define setpbit(p,b)           sbi(p, b)
# define pbitone(p,b)           bit_is_set(p,b)
# define pbitzero(p,b)          bit_is_clear(p,b)

#define ENABLE_ALL_IRQS()       __asm__ __volatile__ ("sei")
#define DISABLE_ALL_IRQS()      __asm__ __volatile__ ("cli")

#define BMSK(a)			(1<<(a))

/*-------------------------------- Konstanten -------------------------------*/
#define HM_DATA_PORT	PORTA
#define HM_DATA_DIR	DDRA
#define HM_DATA_INPUT	PINA

#define HM_CTRL_PORT	PORTC
#define HM_CTRL_DIR	DDRC
#define HM_CTRL_INPUT	PINC

#define HM_CH1_CH2		0	/* output */
#define HM_RES_AZ		1	/* output */
#define HM_CLK_AZ		2	/* output */
#define HM_SRQ			3	/* output */
#define HM_RW			4	/* input, unused */
#define HM_TE			5	/* input */
#define	HM_RESET		6	/* output */
#define HM_REMOTE		7	/* input, unused */

#define WDOG

/*----------------------------------- Oscillator frequency ------------------*/
#define CLK_FREQ	7372800ul


#ifndef CLK_FREQ
#  if defined AVR_AT90S2313 
#    define CLK_FREQ	10000000ul
#  elif (defined AVR_AT90S8515 || defined AVR_AT90S2333)
#    define CLK_FREQ	8000000ul
#  else
#    define CLK_FREQ	4000000ul
#  endif
#endif

#ifndef WDOG
#  define WDOG_RESET()
#else
#  define WDOG_RESET() wdt_reset()
#endif

#define NO_ERR			0
#define ERR_TE_INACTIVE		1
#define ERR_TIME_OUT		2
#define ERR_INTERRUPT		3

#define NUMBER_OF_SAMPLES	1024

/*-------------------------------- types ------------------------------------*/
typedef unsigned char   UINT8_T;
typedef unsigned short  UINT16_T;
typedef unsigned long   UINT32_T; 

/*-------------------------------- Kommandos --------------------------------*/
#define HMCMD_INIT		'I'
const char CmdInit[]		PROGMEM = "Initialize";
#define HMCMD_IDENTIFY		'i'
const char CmdIdentify[]	PROGMEM = "Identify";
#define HMCMD_STATUS		's'
const char CmdStatus[] 		PROGMEM = "Status";
#define HMCMD_READ_DATA		'd'
const char CmdReadData[] 	PROGMEM = "Daten lesen";
#define HMCMD_RESET		'r'
const char CmdReset[] 		PROGMEM = "Reset (Single Shot)";
#define HMCMD_WAIT_TE		't'
const char CmdWaitTE[] 		PROGMEM = "Wait for TE (Single Shot)";
#define HMCMD_CH1_SEL		'1'
const char CmdCh1[] 		PROGMEM = "Select Ch. 1";
#define HMCMD_CH2_SEL		'2'
const char CmdCh2[] 		PROGMEM = "Select Ch. 2";
#define HMCMD_CH_UNSEL		'0'
const char CmdChUnsel[]		PROGMEM = "Unselect Channel";
#define HMCMD_RELEASE		'e'
const char CmdChRelease[]	PROGMEM = "Release Oscilloscope";

/*-------------------------------- Makros -----------------------------------*/
#define HM_RES_AZ_ACTIVE()	clrpbit(HM_CTRL_PORT, HM_RES_AZ)
#define HM_RES_AZ_INACTIVE()	setpbit(HM_CTRL_PORT, HM_RES_AZ)

#define HM_CLK_AZ_ACTIVE()	clrpbit(HM_CTRL_PORT, HM_CLK_AZ)
#define HM_CLK_AZ_INACTIVE()	setpbit(HM_CTRL_PORT, HM_CLK_AZ)

#define HM_SRQ_ACTIVE()		setpbit(HM_CTRL_PORT, HM_SRQ)
#define HM_SRQ_INACTIVE()	clrpbit(HM_CTRL_PORT, HM_SRQ)

#define	HM_RESET_ACTIVE()	clrpbit(HM_CTRL_PORT, HM_RESET)
#define	HM_RESET_INACTIVE()	setpbit(HM_CTRL_PORT, HM_RESET)

#define HM_CHSEL_CH1()		setpbit(HM_CTRL_PORT, HM_CH1_CH2)
#define HM_CHSEL_CH2()		clrpbit(HM_CTRL_PORT, HM_CH1_CH2)

#define HM_CHSEL_OUTPUT()	setpbit(HM_CTRL_DIR, HM_CH1_CH2)

#define HM_CHSEL_INPUT()	do{	clrpbit(HM_CTRL_DIR, HM_CH1_CH2); \
					clrpbit(HM_CTRL_PORT, HM_CH1_CH2);\
				}while(0)

#define HM_TE_ACTIVE()		bit_is_clear(HM_CTRL_INPUT, HM_TE)

#define RTS_ACTIVE()		clrpbit(RS_CTRL_PORT, RS_RTS_OUT)
#define RTS_INACTIVE()		setpbit(RS_CTRL_PORT, RS_RTS_OUT)

/*-------------------------------- Types ------------------------------------*/
typedef UINT16_T 	TIMER_T;
#define MAX_TIMER	0xFFFFl

/*-------------------------------- Global Variables -------------------------*/
const char Msg[] 	PROGMEM = "\r\nhm205_2, Ver.:" VERSION 			\
						"(Built:"__DATE__ "/"__TIME__")"\
						" V.Bosch\r\n";
const char BaudMsg1[]		PROGMEM = "Baudrate: ";
const char BaudMsg2[]		PROGMEM = "Bd";
const char BaudDivMsg1[]	PROGMEM = ", (Teiler:";
const char BaudDivMsg2[]	PROGMEM = " dec)\r\n";
const char CtsHskMsg[]		PROGMEM = "CTS/RTS-";
const char NoHskMsg[]		PROGMEM = "no";
const char HskMsg[]		PROGMEM = " handshake\r\n";
const char IndentMsg[]		PROGMEM = "     ";
const char DotsMsg[] 		PROGMEM = "...........";
const char NewLineMsg[]		PROGMEM = "\r\n";
const char CmdMsg[]		PROGMEM = "\r\nKommandos:\r\n";

static volatile TIMER_T SysTime = 0x00;

/*---------------------------------- Baud rate -------------------------------*/
#define BAUD_DIVISOR(a)	(UINT8_T)((CLK_FREQ + 8 * (a)) / (16 * (a)) - 1)
#define BAUD_RATE(a)	(CLK_FREQ/(16 * (a) + 1))

/*---------------------------------- Handshake signals -----------------------*/
#define RS_CTRL_PORT		PORTD
#define RS_CTRL_DIR		DDRD
#define RS_CTRL_INPUT		PIND

#define RS_RXD			0
#define RS_TXD			1
#define RS_RTS_OUT		2
#define RS_CTS_IN		3

#define RTS_ACTIVE()		clrpbit(RS_CTRL_PORT, RS_RTS_OUT)
#define RTS_INACTIVE()	 	setpbit(RS_CTRL_PORT, RS_RTS_OUT)

#define RTS_IS_ACTIVE()  	bit_is_clear(RS_CTRL_INPUT, RS_RTS_OUT)
#define RTS_IS_INACTIVE() 	bit_is_set(RS_CTRL_INPUT, RS_RTS_OUT)

#define CTS_IS_ACTIVE() 	bit_is_clear(RS_CTRL_INPUT, RS_CTS_IN)
#define CTS_IS_INACTIVE() 	bit_is_set(RS_CTRL_INPUT, RS_CTS_IN)

/* Trigger on falling edge */
#define IRPT_CTS_BECOMES_ACTIVE()   portand(MCUCR, ~BMSK(ISC10)) 
/* Trigger on rising edge */
#define IRPT_CTS_BECOMES_INACTIVE() portor(MCUCR,   BMSK(ISC10)) 

#define CTS_IRPT_INIT()	do{	clrpbit(RS_CTRL_DIR, RS_CTS_IN); /* CTS is input */\
			 	portor(MCUCR, BMSK(ISC11));  /* trigger on edges */\
			 	if(CTS_IS_INACTIVE()) IRPT_CTS_BECOMES_ACTIVE();   \
			 	else IRPT_CTS_BECOMES_INACTIVE();		   \
			 	portor(GIMSK,BMSK(INT1)); /* enable CTS-Irpt */	   \
			 }while(0)
	
#define RTS_INIT()	do{	setpbit(RS_CTRL_DIR, RS_RTS_OUT);	\
			 	RTS_ACTIVE();				\
			}while(0) 

#define CFG_PORT		PORTD
#define CFG_DIR			DDRD
#define CFG_INPUT		PIND

#define CFG_CTS			4
#define CFG_BAUD_MASK		0xE0	/* Mask out baudrate config bits */
#define CFG_BAUD_LSB		5	/* bit pos. of lsb of baudrate config bits */ 


#define CFG_CTS_ACTIVE() bit_is_clear(CFG_INPUT, CFG_CTS)

#define CFG_BAUD()	 ((~(inb(CFG_INPUT)) & CFG_BAUD_MASK) >> CFG_BAUD_LSB)

#define CFG_INIT()	do{portand(CFG_DIR, ~(CFG_BAUD_MASK | BMSK(CFG_CTS))); 	\
			 	portor(CFG_PORT, CFG_BAUD_MASK | BMSK(CFG_CTS));\
			}while(0)
							
/*------------------------ UART Buffer Defines-------------------------------*/ 
#define Rs232RX_BUFFER_SIZE  32		/* 2,4,8,16,32,64,128 or 256 bytes */ 
#define Rs232RX_BUFFER_MASK (Rs232RX_BUFFER_SIZE - 1) 

#if ((Rs232RX_BUFFER_SIZE & Rs232RX_BUFFER_MASK) != 0) 
#  error RX buffer size is not a power of 2 
#endif

#define Rs232TX_BUFFER_SIZE  256	/* 2,4,8,16,32,64,128 or 256 bytes */ 
#define Rs232TX_BUFFER_MASK (Rs232TX_BUFFER_SIZE - 1) 

#if ((Rs232TX_BUFFER_SIZE & Rs232TX_BUFFER_MASK) != 0)
#  error TX buffer size is not a power of 2 
#endif

#if ((Rs232RX_BUFFER_SIZE + Rs232TX_BUFFER_SIZE) >= (RAMEND - 0x5F))
#  error size of transmit and receive buffer exceedes size of sram
#endif

/*------------------------ RTS trigger levels ------------------------------*/
#define RTS_ACTIVE_LEVEL	(Rs232RX_BUFFER_SIZE / 2)

#if (Rs232RX_BUFFER_SIZE / 4 >= 16) 
#  define RTS_INACTIVE_LEVEL	(Rs232RX_BUFFER_SIZE / 2)	
#else
#  define RTS_INACTIVE_LEVEL    (Rs232RX_BUFFER_SIZE / 4)
#endif

/*------------------------ Static Variables --------------------------------*/ 
static char Rs232RxBuf[Rs232RX_BUFFER_SIZE]; 
static volatile UINT8_T Rs232RxHead; 
static volatile UINT8_T Rs232RxTail;
static volatile UINT8_T Rs232RxRtsTrigger;

static char Rs232TxBuf[Rs232TX_BUFFER_SIZE]; 
static volatile UINT8_T Rs232TxHead; 
static volatile UINT8_T Rs232TxTail;

static volatile UINT8_T Rs232ErrorFlags = 0;
#define Rs232REC_OVERFLOW			BMSK(0)

UINT8_T BaudDiv;

/*-------------------------------------------------------------------
 * Timer0 
 *---------------------------------------------------------------------------*/
 #define T0STOP                          0
 #define T0DIV1                          1
 #define T0DIV8                          2       
 #define T0DIV64                         3
 #define T0DIV256                        4
 #define T0DIV1204                       5
 #define T0FALLINGEDGE                   6
 #define T0RISINGEDGE                    7

#define T0DIV 				T0DIV64 

#if (T0DIV == T0STOP)
#  define T0DIVVAL                      0xFFFFFFFF
#elif (T0DIV == T0DIV1)
#  define T0DIVVAL                         1
#elif (T0DIV == T0DIV8)
#  define T0DIVVAL                         8
#elif (T0DIV == T0DIV64)
#  define T0DIVVAL                        64
#elif (T0DIV == T0DIV256)
#  define T0DIVVAL                       256
#elif (T0DIV == T0DIV1204)
#  define T0DIVVAL                      1024
#elif (T0DIV == T0FALLINGEDGE)
#  define T0FREQ                           0
#elif (T0DIV == T0RISINGEDGE)
#  define T0FREQ                           0
#else
#  error Invalid T0 Prescale selected  
#endif

#define TIMER0_FREQ	((CLK_FREQ)/T0DIVVAL)
#define TIMER0_OVL_FREQ	(TIMER0_FREQ/256)

#define T0_START()                      portout(TCCR0, T0DIV)
#define T0_STOP()                	portout(TCCR0, T0STOP)

#define T0_IRQ_DISABLE()                portand(TIMSK, ~BMSK(TOIE0))
#define T0_IRQ_ENABLE()                 portor(TIMSK, BMSK(TOIE0))
#define T0_IRQ_IS_ENABLED()             ((portin(TIMSK) & BMSK(TOIE0)) != 0)
#define T0_CLR_IRQ_FLAG()               portout(TIFR, BMSK(TOV0))
#define T0_IRQ_FLAG_IS_SET()            ((portin(TIFR) & BMSK(TOV0)) != 0)

#define TIME_OUT_WAITING_TE	((TIMER_T)51 * (TIMER_T)TIMER0_OVL_FREQ)

/*-----------------------------------------------------------
 * interrupt handler: Receiver
 *-------------------------------------------------------------------------*/
SIGNAL(SIG_UART_RECV) {   			/* Rs232RX_vect */
  char data;   
  UINT8_T tmphead;

  data = inb(UDR); 				/* read the received data */
  if(Rs232RxRtsTrigger == Rs232RxHead) {
    while(bit_is_clear(USR, RXC)) WDOG_RESET();
    RTS_INACTIVE();
  }
  tmphead = (Rs232RxHead + 1) & Rs232RX_BUFFER_MASK;	/* calc. buffer idx */ 
  Rs232RxHead = tmphead;				/* store new index */
  if(tmphead == Rs232RxTail) {
    Rs232ErrorFlags |= Rs232REC_OVERFLOW;
  }
  Rs232RxBuf[tmphead] = data; 			/* store data in buffer */ 
}

/*-----------------------------------------------------------
 * interrupt handler: Transmitter
 *-------------------------------------------------------------------------*/
SIGNAL(SIG_UART_DATA) {				/* Rs232UDRE_vect */   
  UINT8_T tmptail;
  							   
  if(Rs232TxHead != Rs232TxTail) {		     	/* all data transmitted? */
    tmptail = (Rs232TxTail + 1) & Rs232TX_BUFFER_MASK;	/* calc. buffer index */
    Rs232TxTail = tmptail; 				/* store new index */
    portout(UDR, Rs232TxBuf[tmptail]);			/* start transmition */   
  } else {
    clrpbit(UCR, UDRIE); 					/* disable UDRE interrupt */   
  } 
}

/*-----------------------------------------------------------
 * interrupt handler: CTS
 *-------------------------------------------------------------------------*/
SIGNAL(SIG_INTERRUPT1) {				/* INT1_vect */ 

  if(CTS_IS_INACTIVE()) {
    IRPT_CTS_BECOMES_ACTIVE();
    clrpbit(UCR, UDRIE);				/* disable UDRE interrupt */
  } else {
    IRPT_CTS_BECOMES_INACTIVE();
    portout(UDR, 0x55); 
    while(bit_is_clear(USR, TXC)) WDOG_RESET();
    if(Rs232TxHead != Rs232TxTail) { 
      setpbit(UCR, UDRIE);			/* enable UDRE interrupt */ 
    }
  }
}

/*----------------------------------------------------------------
 * initialize UART 
 *--------------------------------------------------------------------------*/ 
static void Rs232Init(void) {   
  UINT8_T BaudIdx;

  do{
    BaudIdx = CFG_BAUD();
    WDOG_RESET();
  }while(BaudIdx != CFG_BAUD());
  switch(BaudIdx) {
    case 7: BaudDiv = BAUD_DIVISOR(230400ul); 	break;
    case 6: BaudDiv = BAUD_DIVISOR(115200ul); 	break;
    case 5: BaudDiv = BAUD_DIVISOR(57600ul); 	break;
    case 4: BaudDiv = BAUD_DIVISOR(38400ul); 	break;
    case 3: BaudDiv = BAUD_DIVISOR(19200ul); 	break;
    case 2: BaudDiv = BAUD_DIVISOR(9600ul); 	break;
    case 1: BaudDiv = BAUD_DIVISOR(4800ul);	break;
    case 0: BaudDiv = BAUD_DIVISOR(2400ul); 	break;
  } /* switch(BaudIdx) */
  portout(UBRR, BaudDiv);
  /*------- enable UART receiver and transmitter, and receive interrupt ----*/   
  portout(UCR, BMSK(RXCIE)|BMSK(RXEN)|BMSK(TXEN));
  /*-------------------- flush buffers -------------------------------------*/
  Rs232RxTail = Rs232RxHead = Rs232TxTail = Rs232TxHead = 0;
  Rs232RxRtsTrigger = RTS_INACTIVE_LEVEL;
  /*-------------------- handshake signals ---------------------------------*/
  if(CFG_CTS_ACTIVE()) CTS_IRPT_INIT();
  RTS_INIT();
}

/*-------------------------------------------------------------------------
 * Read function 
 *--------------------------------------------------------------------------*/
static char Rs232In(void) {   
  UINT8_T tmptail;
  
  while(Rs232RxHead == Rs232RxTail) WDOG_RESET();	/* wait for data */
  tmptail = (Rs232RxTail + 1) & Rs232RX_BUFFER_MASK;	/* calculate buffer idx */
  Rs232RxTail = tmptail; 				/* store new index */
  Rs232RxRtsTrigger = (Rs232RxRtsTrigger + 1) & Rs232RX_BUFFER_MASK;
  if(RTS_IS_INACTIVE()) {
    if(((tmptail + Rs232RX_BUFFER_SIZE - Rs232RxTail) & Rs232RX_BUFFER_MASK) <= \
    						RTS_ACTIVE_LEVEL) {
      RTS_ACTIVE();     
    }
  }
  return Rs232RxBuf[tmptail]; 				/* return data */ 
}

/*-------------------------------------------------------------------------
 * write function 
 *--------------------------------------------------------------------------*/
static void Rs232Out(char data) {   
  UINT8_T tmphead;
  			   				   
  tmphead = (Rs232TxHead + 1) & Rs232TX_BUFFER_MASK; 	/* calculate buffer index */
  while(tmphead == Rs232TxTail) WDOG_RESET();		/* free space in buffer? */
  Rs232TxBuf[tmphead] = data; 				/* store data in buffer */   
  Rs232TxHead = tmphead; 				/* store new index */
  if(!CFG_CTS_ACTIVE()) {
    setpbit(UCR, UDRIE);				/* enable UDRE interrupt */
  } else {
    if(CTS_IS_ACTIVE()) setpbit(UCR, UDRIE); 		/* enable UDRE interrupt */ 
  }
}

/*----------------------------------------------------------------------
 *
 *---------------------------------------------------------------------------*/
static void Rs232OutString(const char *s) {
  char c;
  while((c = (char)prgmem(s++)) != '\0') {
    Rs232Out((char)c);
    WDOG_RESET();
  }
}

/*---------------------------------------------------------------------
 *
 *---------------------------------------------------------------------------*/
static void Rs232ErrChk(void) {
  static const char OvflwErrMsg[] PROGMEM = "\r\n\n***** Receiver overflow *****\r\n\n";
  if(Rs232ErrorFlags == 0) return;
  if(Rs232ErrorFlags & Rs232REC_OVERFLOW) {
    Rs232OutString(OvflwErrMsg);    
  }
  Rs232ErrorFlags = 0;
}

/*--------------------------------------------------------------------
 * return 0 (FALSE) if the receive buffer is empty 
 *--------------------------------------------------------------------------*/ 
static char Rs232InStat( void ) {
  return (Rs232RxHead != Rs232RxTail); 	
}

/*-------------------------------------------------------------------------
 * return 0 (FALSE) if transmit buffer might store new data
 *--------------------------------------------------------------------------*/
static char Rs232OutStat(void) {   
  UINT8_T tmphead;
  			   				   
  tmphead = (Rs232TxHead + 1) & Rs232TX_BUFFER_MASK; 	/* calculate buffer index */
  return(tmphead != Rs232TxTail);		
}

/*----------------------------------------------------------------------
 * Wait until transmitter buffer is empty
 *---------------------------------------------------------------------------*/
static void Rs232Flush(void) {
  while(Rs232TxHead != Rs232TxTail) WDOG_RESET();
}  

/*----------------------------------------------------------------------
 *
 *---------------------------------------------------------------------------*/
static void Rs232UDec(UINT32_T Val) {
  UINT32_T LTmp;
  
  LTmp = 1;
  WDOG_RESET();
  while(LTmp <= Val) LTmp *= 10;
  LTmp /= 10;
  while(1) {
    Rs232Out((char)(Val / LTmp) + '0');
    Val %= LTmp;
    LTmp /= 10;
    if(LTmp == 0) return;
    WDOG_RESET();
  } /* while(1) */
}

/*----------------------------------------------------------------------
 *
 *---------------------------------------------------------------------------*/
static void Rs232CmdUsageOut(char Cmd, const char *s) {
  Rs232OutString(IndentMsg);
  Rs232Out(Cmd);
  Rs232OutString(DotsMsg);
  Rs232OutString(s);
  Rs232OutString(NewLineMsg);
}

/*--------------------------------------------------------------------- 
 *
 *---------------------------------------------------------------------------*/
static void Delay(UINT8_T Cycles) {
  UINT8_T i;
  
  for(i = 0; i < Cycles; i++) { 
#ifndef WDOG
    asm volatile ("nop");
#else
    WDOG_RESET();
#endif
  }
}

/*---------------------------------------------------------------------
 * Timer-0-ISR
 *
 *---------------------------------------------------------------------------*/
SIGNAL(SIG_OVERFLOW0) {
  SysTime++;
}

/*-------------------------------------------------------------------
 * Auslesen des Systemtimers.
 * Ist der Uebergabewert 0, so wird der aktuelle Stand des Systemtimers 
 * zurueckgegeben, ist er ungleich null, so wird die Differenz zwischen dem
 * aktuellen Stand des Systemtimers und dem uebergebenen Wert zurueckgegeben,
 * wobei zwischenzeitlich erfolgte Ueberlaeufe korrekt behandelt werden.
 *
 *---------------------------------------------------------------------------*/
static TIMER_T GetTimer(TIMER_T LastTime) {
  TIMER_T TimeBuffer;

  do{
    TimeBuffer = SysTime;
    WDOG_RESET();
  }while(TimeBuffer != SysTime);
  if(LastTime == 0) return(TimeBuffer);
  return((LastTime <= TimeBuffer) ? TimeBuffer - LastTime :     \
  				(MAX_TIMER - LastTime) + TimeBuffer);
}

/*---------------------------------------------------------------------
 * Busy-Waiting!
 *---------------------------------------------------------------------------*/
static void WaitTTicks(TIMER_T Ticks) {
  TIMER_T StartTime;
    
  StartTime = GetTimer(0);
  while(GetTimer(StartTime) < Ticks) WDOG_RESET();
}

/*---------------------------------------------------------------------
 *
 *--------------------------------------------------------------------------*/
static char Nibble2Hex(char c) {
  c &= 0x0F;
  if(c < 10) return(c + '0');
  return(c - 10 + 'a');
}

/*----------------------------------------------------------------------
 * send out one byte as 2byte-hex-value on the rs 232 interface 
 *---------------------------------------------------------------------------*/
static void Rs232HexOut(char c) {
 
  Rs232Out(Nibble2Hex(c >> 4));
  Rs232Out(Nibble2Hex(c));
}

/*---------------------------------------------------------------------
 * initialize data port:
 *	Input, 
 *	Pullups active
 * initialize control port:
 *
 *---------------------------------------------------------------------------*/
static void HamegInit(void) {

  /*------------- data port -----------------------------------*/
  portout(HM_DATA_PORT, 0xFF);	/* Pull-ups active */
  /*------------- set output signals incative -----------------*/
  HM_SRQ_INACTIVE();
  HM_RES_AZ_INACTIVE();
  HM_CLK_AZ_INACTIVE();
  HM_RESET_INACTIVE();  
  /*------------- data direction control port -----------------*/
  portor(HM_CTRL_DIR, 	BMSK(HM_RES_AZ)	|	\
  			BMSK(HM_CLK_AZ)	|	\
  			BMSK(HM_SRQ)	|	\
  			BMSK(HM_RESET)		);
  /*------------- ch1/ch2 is input ----------------------------*/
  HM_CHSEL_INPUT();
  /*------------- pull up inputs and unused pins --------------*/
  portor(HM_CTRL_PORT, BMSK(HM_RW)| BMSK(HM_REMOTE) | BMSK(HM_TE));

} 

/*--------------------------------------------------------------------
 *
 *--------------------------------------------------------------------------*/
static void Status(void) {
  Rs232HexOut(inb(HM_CTRL_INPUT));
}

/*----------------------------------------------------------------------
 *
 *--------------------------------------------------------------------------*/
static void HamegRead(void) {
  UINT16_T i;

  if(! HM_TE_ACTIVE()) {
    HamegInit();
    Rs232Out(ERR_TE_INACTIVE);
    return;
  }
  HM_SRQ_ACTIVE();
  HM_CLK_AZ_INACTIVE();
  Delay(1);
  HM_RES_AZ_ACTIVE();
  Delay(10);
  HM_RES_AZ_INACTIVE();
  Rs232HexOut(inb(HM_DATA_INPUT));
  HM_CLK_AZ_ACTIVE();
  for(i = 0; i < NUMBER_OF_SAMPLES - 1; i++) {
    HM_CLK_AZ_INACTIVE();
    Rs232HexOut(inb(HM_DATA_INPUT));
    HM_CLK_AZ_ACTIVE();
    WDOG_RESET();
  } /* for(i = 0; i < NUMBER_OF_SAMPLES; i++) */
  HM_CLK_AZ_INACTIVE();
  Delay(1);
  HamegInit();
}

/*-----------------------------------------------------------------
 *
 *------------------------------------------------------------------------*/
static void HamegReset(void) {
  if(! HM_TE_ACTIVE()) {
    HamegInit();
    Rs232Out(ERR_TE_INACTIVE);
    return;
  }
  HM_RESET_ACTIVE();  
  Delay(10);
  HM_RESET_INACTIVE();  
}

TIMER_T TimeOut = TIME_OUT_WAITING_TE; 
/*-----------------------------------------------------------------
 *
 *------------------------------------------------------------------------*/
static void HamegWaitTe(void){
  register TIMER_T StartTime;
  
  StartTime = GetTimer(0);
  while(!HM_TE_ACTIVE()){ 
    WDOG_RESET();
    if(GetTimer(StartTime) >= TimeOut){
      Rs232Out(ERR_TIME_OUT);
      return;
    }
    if(Rs232InStat() != 0){
      Rs232In();
      Rs232Out(ERR_INTERRUPT);
      return;
    }
  }
  Rs232Out(toupper(HMCMD_WAIT_TE));
}


/*-------------------------------------------------------------------
 *
 *------------------------------------------------------------------------*/
static void HamegChSel(char Channel) {
  switch(Channel) {
    case 0: HM_CHSEL_INPUT();
      break;
    case 1: 	HM_CHSEL_OUTPUT();
    		HM_CHSEL_CH1();
      break;
    case 2: 	HM_CHSEL_OUTPUT();
    		HM_CHSEL_CH2();
      break;
  } /* switch(Channel) */
}

/*----------------------------------------------------------------------
 *
 *---------------------------------------------------------------------------*/
static void Usage(void) {
  Rs232OutString(Msg);
  Rs232OutString(CmdMsg);
  Rs232CmdUsageOut(HMCMD_STATUS, 	CmdStatus);
  Rs232CmdUsageOut(HMCMD_READ_DATA, 	CmdReadData);
  Rs232CmdUsageOut(HMCMD_RESET, 	CmdReset);
  Rs232CmdUsageOut(HMCMD_WAIT_TE, 	CmdWaitTE);
  Rs232CmdUsageOut(HMCMD_CH1_SEL, 	CmdCh1);
  Rs232CmdUsageOut(HMCMD_CH2_SEL, 	CmdCh2);
  Rs232CmdUsageOut(HMCMD_CH_UNSEL, 	CmdChUnsel);
  Rs232CmdUsageOut(HMCMD_RELEASE, 	CmdChRelease);
  Rs232CmdUsageOut(HMCMD_INIT,		CmdInit);
  Rs232CmdUsageOut(HMCMD_IDENTIFY,	CmdIdentify);
} 

/*--------------------------------------------------------
 *
 *----------------------------------------------------------------------------*/
static void Identify(void) {
  /*----------------- Message -----------------------------------------------*/
  Rs232OutString(Msg);
  Rs232OutString(BaudMsg1);
  Rs232UDec(BAUD_RATE(BaudDiv));
  Rs232OutString(BaudMsg2);
  Rs232OutString(BaudDivMsg1);
  Rs232UDec(BaudDiv);
  Rs232OutString(BaudDivMsg2);
  if(CFG_CTS_ACTIVE()) {
    Rs232OutString(CtsHskMsg);
  } else {
    Rs232OutString(NoHskMsg);
  }
  Rs232OutString(HskMsg);
}

/*--------------------------------------------------------
 *
 *----------------------------------------------------------------------------*/
static void Initialize(void) {
  HamegInit();
  CFG_INIT();
  Rs232Init();
  /*----------------- Watchdog ----------------------------------------------*/
# ifdef WDOG
  wdt_enable(7);
  WDOG_RESET();  
# endif
  /*----------------- Timer0 ------------------------------------------------*/
  portor(TIMSK, BMSK(TOIE0)); 
  T0_START();
  /*----------------- Interrupt ---------------------------------------------*/
  ENABLE_ALL_IRQS();
  Identify();
}

/*----------------------------------------------------------------------
 *
 *---------------------------------------------------------------------------*/
int main(void) {
  char c;
      
  Initialize();
  /*----------------- Loop --------------------------------------------------*/
  while(1) {
    if(Rs232InStat() != 0) {
      c = Rs232In();
      Rs232Out(c);
      switch(c) {
        case HMCMD_INIT: 
        	Initialize();
          break;
        case HMCMD_IDENTIFY:
        	Identify();
          break;
        case HMCMD_STATUS: 
        	Status();
	    	Rs232Out('\r');
  		Rs232Out('\n');
          break;
        case HMCMD_READ_DATA: 
        	HamegRead();
	  	Rs232Out('\r');
  		Rs232Out('\n');
          break;
        case HMCMD_RESET: 
        	HamegReset();
		Rs232Out(toupper(HMCMD_RESET));
	  	Rs232Out('\r');
  		Rs232Out('\n');
          break;
        case HMCMD_WAIT_TE: 
        	HamegWaitTe();
	  	Rs232Out('\r');
  		Rs232Out('\n');
          break;
        case HMCMD_CH1_SEL: 
        	HamegChSel(1);
		Rs232Out(HMCMD_CH1_SEL);
	  	Rs232Out('\r');
  		Rs232Out('\n');
          break;
        case HMCMD_CH2_SEL: 
        	HamegChSel(2);
        	Rs232Out(HMCMD_CH2_SEL);
	  	Rs232Out('\r');
  		Rs232Out('\n');
          break;
        case HMCMD_CH_UNSEL: 
        	HamegChSel(0);
        	Rs232Out(HMCMD_CH_UNSEL);
	  	Rs232Out('\r');
  		Rs232Out('\n');
          break;
        case HMCMD_RELEASE: 
        	HamegInit();
		Rs232Out(toupper(HMCMD_RELEASE));
		Rs232Out('\r');
		Rs232Out('\n');
          break;
        default: Usage();    
      } /* switch(c = Rs232In()) */
    } /* if(Rs232InStat() != 0)  */
    Rs232ErrChk();
    WDOG_RESET();
  } /* while(1) */
  return(0);
}
